@@ -28,15 +28,19 @@ from __future__ import unicode_literals, absolute_import
 
 import os
 # import re
+import datetime
+import shutil
 import tempfile
 import warnings
 import logging
 
 import six
+from mako.template import Template
 
 from rattail.util import (load_object, load_entry_points,
-                          OrderedDict, progress_loop, pretty_quantity)
-from rattail.files import temp_path
+                          OrderedDict, progress_loop,
+                          pretty_quantity, pretty_hours)
+from rattail.files import temp_path, resource_path
 from rattail.mail import send_email
 from rattail.config import parse_list
 from rattail.core import get_uuid, Object
@@ -69,6 +73,8 @@ class AppHandler(object):
         'vendors': 'rattail.autocomplete.vendors:VendorAutocompleter',
     }
 
+    setting_utctime_format = '%Y-%m-%d %H:%M:%S'
+
     def __init__(self, config):
         self.config = config
 
@@ -197,6 +203,40 @@ class AppHandler(object):
         value = session.execute(sql).scalar()
         return value
 
+    def get_setting(self, session, name, typ=None, **kwargs):
+        model = self.model
+        setting = session.query(model.Setting).get(name)
+        value = None if setting is None else setting.value
+
+        if typ == 'utctime':
+            if value:
+                value = datetime.datetime.strptime(value, self.setting_utctime_format)
+            else:
+                value = None
+
+        return value
+
+    def save_setting(self, session, name, value, typ=None, **kwargs):
+        model = self.model
+
+        if typ == 'utctime':
+            if value:
+                value = value.strftime(self.setting_utctime_format)
+            else:
+                value = None
+
+        setting = session.query(model.Setting).get(name)
+        if not setting:
+            setting = model.Setting(name=name)
+            session.add(setting)
+        setting.value = value
+
+    def delete_setting(self, session, name, **kwargs):
+        model = self.model
+        setting = session.query(model.Setting).get(name)
+        if setting:
+            session.delete(setting)
+
     def get_active_stores(self, session, **kwargs):
         """
         Returns the list of "active" stores.  A store is considered
@@ -786,6 +826,106 @@ class AppHandler(object):
         from rattail.db import cache
         return cache.cache_model(session, model, **kwargs)
 
+    def make_appdir(self, path, **kwargs):
+        """
+        Establish an appdir at the given path.  This is really only
+        creating some folders so should be safe to run multiple times
+        even if the appdir already exists.
+        """
+        appdir = path
+        if not os.path.exists(appdir):
+            os.mkdir(appdir)
+
+        folders = [
+            'data',
+            os.path.join('data', 'uploads'),
+            'log',
+            'sessions',
+            'work',
+        ]
+        for name in folders:
+            path = os.path.join(appdir, name)
+            if not os.path.exists(path):
+                os.mkdir(path)
+
+    def make_config_file(self, file_type, output_path, template_path=None,
+                         **kwargs):
+        """
+        Write a new config file of given type to specified location.
+
+        :param file_type: The "type" of config file to create.  This
+           is used to locate the file template, if ``template_path``
+           is not specified.  It also is used as default output
+           filename, if ``output_path`` is a folder.
+
+        :param output_path: Path to which new config file should be
+           written.  If this is a folder, then the filename is deduced
+           from the ``file_type``.
+
+        :param template_path: Optional path to config file template to
+           use.  If not specified, it will be looked up dynamically
+           based on the ``file_type``.  Note that the first template
+           found to match will be used.  Mako (``*.mako``) templates
+           are preferred, otherwise the template is assumed to be
+           "plain" and will be copied as-is to the output path.
+
+        :param **kwargs: Context to be passed to the Mako template, if
+           applicable.
+
+        :returns: Final path to which new config file was written.
+        """
+        # lookup template if not specified
+        if not template_path:
+            template_path = self.find_config_template(file_type)
+            if not template_path:
+                raise RuntimeError("config template not found for type: {}".format(file_type))
+
+        # deduce filename if not specified
+        if os.path.isdir(output_path):
+            output_path = os.path.join(output_path, '{}.conf'.format(file_type))
+
+        # just copy file as-is unless it's mako
+        if not template_path.endswith('.mako'):
+            shutil.copy(template_path, output_path)
+            return output_path
+
+        # render mako template
+        context = {
+            'app_title': "Rattail",
+            'appdir': '/srv/envs/poser',
+            'db_url': 'postresql://user:pass@localhost/poser',
+            'timezone': 'America/Chicago',
+            'pyramid_egg': 'poser',
+            'os': os,
+            'beaker_secret': 'TODO_YOU_SHOULD_CHANGE_THIS',
+            'beaker_key': 'poser',
+            'pyramid_host': '0.0.0.0',
+            'pyramid_port': 9080,
+        }
+        context.update(kwargs)
+        config = Template(filename=template_path).render(**context)
+        with open(output_path, 'wt') as f:
+            f.write(config)
+
+        return output_path
+
+    def find_config_template(self, name):
+        template_paths = self.config.getlist('rattail.config', 'templates',
+                                             default=['rattail:data/config'])
+        for template_path in template_paths:
+
+            # prefer mako templates
+            path = resource_path('{}/{}.conf.mako'.format(template_path.rstrip('/'),
+                                                          name))
+            if os.path.exists(path):
+                return path
+
+            # but plain config works too
+            path = resource_path('{}/{}.conf'.format(template_path.rstrip('/'),
+                                                     name))
+            if os.path.exists(path):
+                return path
+
     def make_temp_dir(self, **kwargs):
         """
         Create a temporary directory.  This is mostly a convenience
@@ -934,12 +1074,13 @@ class AppHandler(object):
 
         :returns: Display string for the value.
         """
-        if value is not None:
-            if value < 0:
-                fmt = "(${{:0,.{}f}})".format(scale)
-                return fmt.format(0 - value)
-            fmt = "${{:0,.{}f}}".format(scale)
-            return fmt.format(value)
+        if value is None:
+            return ''
+        if value < 0:
+            fmt = "(${{:0,.{}f}})".format(scale)
+            return fmt.format(0 - value)
+        fmt = "${{:0,.{}f}}".format(scale)
+        return fmt.format(value)
 
     def render_quantity(self, value, **kwargs):
         """
@@ -1003,6 +1144,25 @@ class AppHandler(object):
         if value is not None:
             return value.strftime('%Y-%m-%d %I:%M:%S %p')
 
+    def render_duration(self, seconds=None, **kwargs):
+        if seconds is None:
+            return ""
+        return pretty_hours(datetime.timedelta(seconds=seconds))
+
+    def render_percent(self, value, places=2, **kwargs):
+        """
+        Render a human-friendly display string for the given
+        percentage value.
+
+        :param value: Should be a decimal representation of the
+           percentage, e.g. ``0.80`` would indicate 80%.
+
+        :param places: Number of decimal places to display in the
+           rendered string.
+        """
+        fmt = '{{:0.{}f}} %'.format(places)
+        return fmt.format(value * 100)
+
     def send_email(self, key, data={}, **kwargs):
         """
         Send an email message of the given type.
