@@ -546,39 +546,8 @@ class CustomerOrderBatchHandler(BatchHandler):
         """
         Return a list of UOM choices for the given product.
         """
-        choices = []
-
-        # Each
-        if not product or not product.weighed:
-            unit_name = self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_EACH]
-            choices.append({'key': self.enum.UNIT_OF_MEASURE_EACH,
-                            'value': unit_name})
-
-        # Pound
-        if not product or product.weighed:
-            unit_name = self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_POUND]
-            choices.append({
-                'key': self.enum.UNIT_OF_MEASURE_POUND,
-                'value': unit_name,
-            })
-
-        # Case
-        case_text = None
-        case_size = self.get_case_size_for_product(product)
-        if case_size is None:
-            case_text = "{} (&times; ?? {})".format(
-                self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_CASE],
-                unit_name)
-        elif case_size > 1:
-            case_text = "{} (&times; {} {})".format(
-                self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_CASE],
-                self.app.render_quantity(case_size),
-                unit_name)
-        if case_text:
-            choices.append({'key': self.enum.UNIT_OF_MEASURE_CASE,
-                            'value': case_text})
-
-        return choices
+        products_handler = self.app.get_products_handler()
+        return products_handler.get_uom_choices(product)
 
     def why_not_add_product(self, product, batch):
         """
