@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -62,6 +62,7 @@ class BatchHandler(object):
     repopulate_when_refresh = False
 
     execute_with_versioning = True
+    delete_with_versioning = True
 
     def __init__(self, config):
         self.config = config
@@ -104,6 +105,8 @@ class BatchHandler(object):
             return self.refresh_with_versioning
         if action == 'execute':
             return self.execute_with_versioning
+        if action == 'delete':
+            return self.delete_with_versioning
         raise NotImplementedError("unknown batch action: {}".format(action))
 
     def is_mutable(self, batch):
@@ -119,6 +122,22 @@ class BatchHandler(object):
             return False
         return True
 
+    def consume_batch_id(self, session, as_str=False):
+        """
+        Consumes a new batch ID from the generator, and returns it.
+
+        :param session: Current session for Rattail DB.
+
+        :param as_str: Flag indicating whether the return value should be a
+           string, as opposed to the default of integer.
+
+        :returns: Batch ID as integer, or zero-padded string of 8 chars.
+        """
+        batch_id = self.app.next_counter_value(session, 'batch_id')
+        if as_str:
+            return '{:08d}'.format(batch_id)
+        return batch_id
+
     def make_basic_batch(self, session, user=None, progress=None, **kwargs):
         """
         Make a new "basic" batch, with no customization beyond what is provided
@@ -132,6 +151,13 @@ class BatchHandler(object):
         kwargs.setdefault('rowcount', 0)
         kwargs.setdefault('complete', False)
 
+        # we used just let the postgres sequence auto-generate the id,
+        # but now we are trying to support more than just postgres,
+        # and so we consume a batch id using shared logic which can
+        # accommodate more than just postgres
+        if 'id' not in kwargs:
+            kwargs['id'] = self.consume_batch_id(session)
+
         # try to provide default creator
         if user and 'created_by' not in kwargs and 'created_by_uuid' not in kwargs:
             kwargs['created_by'] = user
@@ -712,6 +738,23 @@ class BatchHandler(object):
         batch.executed_by = user
         return result
 
+    def get_effective_rows(self, batch):
+        """
+        Should return the set of rows from the given batch which are
+        considered "effective" - i.e. when the batch is executed,
+        these rows should be processed whereas the remainder should
+        not.
+
+        :param batch: A
+           :class:`~rattail.db.model.batch.vendorcatalog.VendorCatalogBatch`
+           instance.
+
+        :returns: List of
+           :class:`~rattail.db.model.batch.vendorcatalog.VendorCatalogBatchRow`
+           instances.
+        """
+        return batch.active_rows()
+
     def execute(self, batch, progress=None, **kwargs):
         """
         Execute the given batch, according to the given kwargs.  This is really
