@@ -147,6 +147,15 @@ class ImportSubcommand(Subcommand):
                             help="List of fields which should be excluded from the import.  "
                             "Any field not listed here, would be included (or not) depending "
                             "on the --fields parameter and/or default importer behavior.")
+        parser.add_argument('--fuzzy-fields',
+                            help="List of fields for which diff comparison should "
+                            "be \"fuzzy\".  This is intended for timestamps and similar "
+                            "values which vary in granularity between systems.")
+        parser.add_argument('--fuzz-factor', type=int, default=1,
+                            help="Numeric value for use with --fuzzy-fields.  For "
+                            "timestamp fields, this refers to the number of seconds "
+                            "by which values are allowed to differ and still be "
+                            "considered a match.  Default fuzz factor is 1.")
 
         # date ranges
         parser.add_argument('--start-date', type=date_argument, metavar='DATE',
@@ -258,6 +267,8 @@ class ImportSubcommand(Subcommand):
             'warnings': args.warnings,
             'fields': parse_list(args.fields),
             'exclude_fields': parse_list(args.exclude_fields),
+            'fuzzy_fields': parse_list(args.fuzzy_fields),
+            'fuzz_factor': args.fuzz_factor,
             'create': args.create,
             'max_create': args.max_create,
             'update': args.update,
