@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -188,13 +188,15 @@ class RattailConfig(object):
 
                 # first bring in any "required" files
                 if parser.has_option('rattail.config', 'require'):
-                    includes = parse_list(parser.get('rattail.config', 'require'))
+                    includes = self.parse_list(
+                        parser.get('rattail.config', 'require'))
                     for included in includes:
                         self.read_file(included, recurse=True, require=True)
 
                 # next try to bring in any "included" files
                 if parser.has_option('rattail.config', 'include'):
-                    includes = parse_list(parser.get('rattail.config', 'include'))
+                    includes = self.parse_list(
+                        parser.get('rattail.config', 'include'))
                     for included in includes:
                         self.read_file(included, recurse=True)
 
@@ -359,9 +361,12 @@ class RattailConfig(object):
         if value is None:
             return None
         if isinstance(value, six.string_types):
-            return parse_list(value)
+            return self.parse_list(value)
         return value            # maybe a caller-provided default?
 
+    def parse_list(self, value):
+        return parse_list(value)
+
     def get_dict(self, section):
         """
         Convenience method which returns a dictionary of options contained
@@ -948,7 +953,7 @@ class ConfigProfile(object):
         return value
 
     def _config_list(self, option):
-        return parse_list(self._config_string(option))
+        return self.config.parse_list(self._config_string(option))
 
     def _config_action(self, name):
         """
