@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -212,6 +212,10 @@ class ConfigExtension(BaseExtension):
             config.setdefault('rattail.importing', 'to_trainwreck.from_trainwreck.export.legacy_handler_setting',
                               'trainwreck.exporting, trainwreck.handler')
 
+            # trainwreck import-self
+            config.setdefault('rattail.importing', 'to_trainwreck.from_self.import.default_cmd',
+                              'trainwreck import-self')
+
             # trainwreck import-trainwreck
             config.setdefault('rattail.importing', 'to_trainwreck.from_trainwreck.import.default_handler',
                               'rattail.trainwreck.importing.trainwreck:FromTrainwreckToTrainwreckImport')
