@@ -27,7 +27,8 @@ def upgrade():
     op.add_column('product', sa.Column('organic', sa.Boolean(), nullable=True))
     product = sa.sql.table('product', sa.sql.column('organic'))
     op.execute(product.update().values({'organic': False}))
-    op.alter_column('product', 'organic', nullable=False)
+    op.alter_column('product', 'organic', nullable=False,
+                    existing_type=sa.Boolean())
     op.add_column('product_version', sa.Column('organic', sa.Boolean(), nullable=True))
 
 
