@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -570,7 +570,10 @@ class ProductCode(Base):
     uuid = uuid_column()
     product_uuid = sa.Column(sa.String(length=32), nullable=False)
     ordinal = sa.Column(sa.Integer(), nullable=False)
-    code = sa.Column(sa.String(length=20), nullable=True)
+
+    code = sa.Column(sa.String(length=20), nullable=True, doc="""
+    Alternate code value for the product, to be used for lookup.
+    """)
 
     def __str__(self):
         return self.code or ''
