@@ -278,6 +278,23 @@ class person_merge_request(Email):
         }
 
 
+class rattail_problems_mailmon_misses(ProblemReportEmail):
+    """
+    Nightly check to alert if incoming messages are noticed to have
+    been "missed" by mailmon daemon.
+    """
+    default_subject = "Mailmon misses"
+    abstract = False
+
+    def sample_data(self, request):
+        app = request.rattail_config.get_app()
+        account = app.make_object(server='example.com')
+        profile = app.make_object(imap_folder='TestFolder')
+        return {
+            'problems': [(account, profile, 1)],
+        }
+
+
 class rattail_problems_stale_inventory_batch(ProblemReportEmail):
     """
     Nightly check for "stale" inventory batches, i.e. those created
