@@ -80,6 +80,96 @@ class MailMonitorDaemon(Daemon):
             time.sleep(.01)
 
 
+class MailmonConnection(object):
+    """
+    Abstraction class to hide IMAP details and expose convenience
+    methods for use with mailmon daemon.
+    """
+
+    def __init__(self, config, account):
+        self.config = config
+        self.account = account
+        self.app = config.get_app()
+
+        self.server = None
+        self.server_recycled = None
+
+    def connect(self):
+
+        # do we already have a connection?
+        if self.server:
+
+            # keep using forever if no recycle delay
+            if not self.account.recycle_delay:
+                return
+
+            # okay so does it need to be recycled?
+            now = self.app.make_utc()
+            if (now - self.server_recycled).seconds < self.account.recycle_delay:
+
+                # nope it's new enough to keep using
+                return
+
+            # too old, should recycle
+            log.debug("recycle time limit reached (%s seconds), "
+                      "disposing of current connection",
+                      self.account.recycle_delay)
+            self.disconnect()
+
+        # establish new connection
+        self.server = imaplib.IMAP4_SSL(self.account.server)
+        result = self.server.login(self.account.username, self.account.password)
+        log.debug("IMAP server login result: %s", result)
+        self.server_recycled = self.app.make_utc()
+
+    def disconnect(self):
+        if not self.server:
+            return
+
+        try:
+            self.server.close()
+        except:
+            log.debug("server.close() failed!", exc_info=True)
+        else:
+            try:
+                self.server.logout()
+            except:
+                log.exception("server.logout() failed!")
+        finally:
+            self.server = None
+
+    def select_folder(self, folder):
+        result = self.server.select(folder)
+        log.debug("IMAP server select (%s) result: %s", folder, result)
+        code, count = result
+        if code != 'OK':
+            raise RuntimeError("unexpected code '{}' from select() of folder: {}".format(
+                code, folder))
+        return int(count[0])
+
+    def get_messages(self, folder, criterion):
+        self.select_folder(folder)
+
+        # log.debug("invoking IMAP4.search()")
+        result = self.server.uid('search', None, criterion)
+        try:
+            code, items = result
+        except:
+            log.exception("unexpected search result for folder %s: %s",
+                          folder, result)
+            raise
+        else:
+            if code != 'OK':
+                raise RuntimeError("IMAP4.search() returned bad code: {}".format(code))
+            return items
+
+    def get_all_messages(self, folder):
+        return self.get_messages(folder, 'ALL')
+
+    def get_unread_messages(self, folder):
+        return self.get_messages(folder, '(UNSEEN)')
+
+
 class IMAPAccountWatcher(object):
     """
     Abstraction to make watching an IMAP account a little more
@@ -94,8 +184,7 @@ class IMAPAccountWatcher(object):
     def __init__(self, config, account):
         self.config = config
         self.account = account
-        self.server = None
-        self.server_recycled = None
+        self.connection = MailmonConnection(config, account)
         self.app = config.get_app()
         self.lastrun_timefmt = get_lastrun_timefmt(self.config)
 
@@ -142,7 +231,7 @@ class IMAPAccountWatcher(object):
 
                         # presumably the error has to do with our server
                         # connection, so make sure we re-connect next time
-                        self.disconnect()
+                        self.connection.disconnect()
 
                         # maybe stop monitoring this profile
                         if profile.stop_on_error:
@@ -169,69 +258,20 @@ class IMAPAccountWatcher(object):
             # and a general pause just to play nice
             time.sleep(0.01)
 
-    def connect(self):
-
-        # do we already have a connection?
-        if self.server:
-
-            # keep using forever if no recycle delay
-            if not self.account.recycle_delay:
-                return
-
-            # okay so does it need to be recycled?
-            now = self.app.make_utc()
-            if (now - self.server_recycled).seconds < self.account.recycle_delay:
-
-                # nope it's new enough to keep using
-                return
-
-            # too old, should recycle
-            log.debug("recycle time limit reached (%s seconds), "
-                      "disposing of current connection",
-                      self.account.recycle_delay)
-            self.disconnect()
-
-        # establish new connection
-        self.server = imaplib.IMAP4_SSL(self.account.server)
-        result = self.server.login(self.account.username, self.account.password)
-        log.debug("IMAP server login result: %s", result)
-        self.server_recycled = self.app.make_utc()
-
-    def disconnect(self):
-        if not self.server:
-            return
-
-        try:
-            self.server.close()
-        except:
-            log.exception("server.close() failed!")
-        else:
-            try:
-                self.server.logout()
-            except:
-                log.exception("server.logout() failed!")
-        finally:
-            self.server = None
-
     def process_profile(self, profile, unread_only=None):
         """
         Poll a particular IMAP folder and process any messages found.
         """
-        self.connect()
-
-        # select the folder
-        result = self.server.select(profile.imap_folder)
-        log.debug("IMAP server select (%s) result: %s",
-                  profile.imap_folder, result)
+        self.connection.connect()
 
         # maybe look for "all" or maybe just "unread"
         if unread_only is None:
             unread_only = profile.imap_unread_only
-        criterion = '(UNSEEN)' if unread_only else 'ALL'
-        # log.debug("invoking IMAP4.search()")
-        code, items = self.server.uid('search', None, criterion)
-        if code != 'OK':
-            raise RuntimeError("IMAP4.search() returned bad code: {}".format(code))
+
+        if unread_only:
+            items = self.connection.get_unread_messages(profile.imap_folder)
+        else:
+            items = self.connection.get_all_messages(profile.imap_folder)
 
         # config may dictacte a "max batch size" in which case we will
         # only process so many messages at a time
@@ -283,7 +323,9 @@ class IMAPAccountWatcher(object):
                       action.spec, attempts, action.retry_attempts, msguid)
 
             try:
-                action.action(self.server, msguid, *action.args, **action.kwargs)
+                # TODO: should not reference IMAP server directly?
+                action.action(self.connection.server, msguid,
+                              *action.args, **action.kwargs)
 
             except:
 
