@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -43,6 +43,25 @@ class ProjectHandler(object):
 
     def __init__(self, config):
         self.config = config
+        self.app = self.config.get_app()
+
+    def get_all_project_types(self):
+        """
+        Returns the list of *all* possible project types.
+        """
+        return [
+            'rattail',
+            'rattail_integration',
+            'tailbone_integration',
+            # 'byjove',
+            'fabric',
+        ]
+
+    def get_supported_project_types(self):
+        """
+        Returns the list of "supported" project types.
+        """
+        return self.get_all_project_types()
 
     def get_storage_dir(self):
         """
@@ -87,8 +106,37 @@ class ProjectHandler(object):
             shutil.rmtree(path)
         os.makedirs(path)
 
+        if project_type == 'rattail_integration':
+
+            if 'name' not in options:
+                options['name'] = options['python_project_name']
+
+            if 'slug' not in options:
+                options['slug'] = options['name']
+
+            if 'year' not in options:
+                options['year'] = self.app.localtime().year
+
+            if 'integration_studly' not in options:
+                words = options['integration_name'].split()
+                options['integration_studly'] = ''.join([word.capitalize()
+                                                         for word in words])
+
+            if 'integration_prefix' not in options:
+                words = options['integration_name'].split()
+                options['integration_prefix'] = '_'.join([word.lower()
+                                                          for word in words])
+
+        if project_type == 'tailbone_integration':
+
+            if 'name' not in options:
+                options['name'] = options['python_project_name']
+
+            if 'year' not in options:
+                options['year'] = self.app.localtime().year
+
         # TODO: all of this logic really belongs elsewhere...
-        if project_type in ('rattail', 'fabric'):
+        if project_type in ('rattail', 'rattail_integration', 'tailbone_integration', 'fabric'):
 
             if 'egg_name' not in options:
                 options['egg_name'] = options['python_project_name'].replace('-', '_')
@@ -228,6 +276,10 @@ class RattailProjectHandler(ProjectHandler):
             return self.generate_byjove_project(slug, options, path)
         elif project_type == 'fabric':
             return self.generate_fabric_project(slug, options, path)
+        elif project_type == 'rattail_integration':
+            return self.generate_rattail_integration_project(slug, options, path)
+        elif project_type == 'tailbone_integration':
+            return self.generate_tailbone_integration_project(slug, options, path)
         else:
             return self.generate_rattail_project(slug, options, path)
 
@@ -490,6 +542,178 @@ class RattailProjectHandler(ProjectHandler):
             self.generate('rattail/machines/server/deploy/poser/logrotate.conf.mako', os.path.join(poser, 'logrotate.conf'),
                           **context)
 
+    def generate_rattail_integration_project(self, slug, options, path):
+        """
+        And here we do some experimentation...
+        """
+        context = options
+
+        ##############################
+        # root project dir
+        ##############################
+
+        self.generate('rattail_integration/gitignore.mako',
+                      os.path.join(path, '.gitignore'),
+                      **context)
+
+        self.generate('rattail_integration/MANIFEST.in.mako',
+                      os.path.join(path, 'MANIFEST.in'),
+                      **context)
+
+        self.generate('rattail_integration/README.rst.mako',
+                      os.path.join(path, 'README.rst'),
+                      **context)
+
+        self.generate('rattail_integration/CHANGELOG.md.mako',
+                      os.path.join(path, 'CHANGELOG.md'),
+                      **context)
+
+        self.generate('rattail_integration/setup.py.mako',
+                      os.path.join(path, 'setup.py'),
+                      **context)
+
+        self.generate('rattail_integration/tasks.py.mako',
+                      os.path.join(path, 'tasks.py'),
+                      **context)
+
+        ##############################
+        # root package dir
+        ##############################
+
+        package = os.path.join(path, options['python_name'])
+        os.makedirs(package)
+
+        self.generate('rattail_integration/package/__init__.py.mako',
+                      os.path.join(package, '__init__.py'),
+                      **context)
+
+        self.generate('rattail_integration/package/_version.py',
+                      os.path.join(package, '_version.py'))
+
+        if context['extends_config']:
+            self.generate('rattail_integration/package/config.py.mako',
+                          os.path.join(package, 'config.py'),
+                          **context)
+
+        ##############################
+        # db package dir
+        ##############################
+
+        if context['extends_db']:
+
+            db = os.path.join(package, 'db')
+            os.makedirs(db)
+
+            self.generate('rattail_integration/package/db/__init__.py',
+                          os.path.join(db, '__init__.py'))
+
+            ####################
+            # model
+            ####################
+
+            model = os.path.join(db, 'model')
+            os.makedirs(model)
+
+            self.generate('rattail_integration/package/db/model/__init__.py.mako',
+                          os.path.join(model, '__init__.py'),
+                          **context)
+
+            self.generate('rattail_integration/package/db/model/customers.py.mako',
+                          os.path.join(model, 'customers.py'),
+                          **context)
+
+            ####################
+            # alembic
+            ####################
+
+            alembic = os.path.join(db, 'alembic')
+            os.makedirs(alembic)
+
+            versions = os.path.join(alembic, 'versions')
+            os.makedirs(versions)
+
+            self.generate('rattail_integration/package/db/alembic/versions/.keepme',
+                          os.path.join(versions, '.keepme'))
+
+    def generate_tailbone_integration_project(self, slug, options, path):
+        """
+        And here we do some experimentation...
+        """
+        context = options
+
+        ##############################
+        # root project dir
+        ##############################
+
+        self.generate('tailbone_integration/gitignore.mako',
+                      os.path.join(path, '.gitignore'),
+                      **context)
+
+        self.generate('tailbone_integration/MANIFEST.in.mako',
+                      os.path.join(path, 'MANIFEST.in'),
+                      **context)
+
+        self.generate('tailbone_integration/README.rst.mako',
+                      os.path.join(path, 'README.rst'),
+                      **context)
+
+        self.generate('tailbone_integration/CHANGELOG.md.mako',
+                      os.path.join(path, 'CHANGELOG.md'),
+                      **context)
+
+        self.generate('tailbone_integration/setup.py.mako',
+                      os.path.join(path, 'setup.py'),
+                      **context)
+
+        self.generate('tailbone_integration/tasks.py.mako',
+                      os.path.join(path, 'tasks.py'),
+                      **context)
+
+        ##############################
+        # root package dir
+        ##############################
+
+        package = os.path.join(path, options['python_name'])
+        os.makedirs(package)
+
+        self.generate('tailbone_integration/package/__init__.py.mako',
+                      os.path.join(package, '__init__.py'),
+                      **context)
+
+        self.generate('tailbone_integration/package/_version.py',
+                      os.path.join(package, '_version.py'))
+
+        ##############################
+        # views
+        ##############################
+
+        views = os.path.join(package, 'views')
+        os.makedirs(views)
+
+        self.generate('tailbone_integration/package/views/__init__.py',
+                      os.path.join(views, '__init__.py'))
+
+        ##############################
+        # static
+        ##############################
+
+        static = os.path.join(package, 'static')
+        os.makedirs(static)
+
+        self.generate('tailbone_integration/package/static/__init__.py.mako',
+                      os.path.join(static, '__init__.py'),
+                      **context)
+
+        ##############################
+        # templates
+        ##############################
+
+        templates = os.path.join(package, 'templates')
+        os.makedirs(templates)
+
+        self.generate('tailbone_integration/package/templates/.keepme',
+                      os.path.join(templates, '.keepme'))
+
     def generate_byjove_project(self, slug, options, path):
         """
         Generate a new 'byjove' project per the given arguments.
