@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -26,4 +26,7 @@ TrainWreck DB model
 
 from __future__ import unicode_literals, absolute_import
 
-from .base import Base, uuid_column, TransactionBase, TransactionItemBase
+from .base import (Base, uuid_column,
+                   TransactionBase,
+                   TransactionItemBase,
+                   TransactionItemDiscountBase)
