@@ -26,12 +26,17 @@ Trainwreck model importers
 
 from __future__ import unicode_literals, absolute_import
 
+from collections import namedtuple
+
 from sqlalchemy import orm
 
 from rattail import importing
 from .util import ToOrFromTrainwreck
 
 
+Discount = namedtuple('Discount', ['discount_type', 'description', 'amount'])
+
+
 class ToTrainwreck(ToOrFromTrainwreck, importing.ToSQLAlchemy):
     """
     Base class for all Trainwreck model importers
@@ -98,6 +103,7 @@ class TransactionItemImporter(ToTrainwreck):
         fields = list(fields)
         fields.extend([
             'transaction_system_id',
+            'discounts',
         ])
         return fields
 
@@ -136,26 +142,41 @@ class TransactionItemImporter(ToTrainwreck):
                                                               query=query,
                                                               key='system_id')
 
-    def normalize_local_object(self, item):
-        data = super(TransactionItemImporter, self).normalize_local_object(item)
-        if data:
-            if 'transaction_system_id' in self.fields:
-                data['transaction_system_id'] = item.transaction.system_id
-            return data
-
     def cache_query(self):
-        trainwreck = self.config.get_trainwreck_model()
         query = super(TransactionItemImporter, self).cache_query()
+        trainwreck = self.config.get_trainwreck_model()
+
+        query = query.join(trainwreck.Transaction)
+
         try:
             system = self.importing_from_system
         except NotImplementedError:
             pass
         else:
             query = query.filter(trainwreck.Transaction.system == system)
+
         time_field = getattr(trainwreck.Transaction, self.match_on_time_field)
-        return query.join(trainwreck.Transaction)\
-                    .filter(time_field >= self.app.make_utc(self.start_time))\
-                    .filter(time_field < self.app.make_utc(self.end_time))
+        query = query.filter(time_field >= self.app.make_utc(self.start_time))\
+                     .filter(time_field < self.app.make_utc(self.end_time))
+
+        if 'discounts' in self.fields:
+            query = query.options(orm.joinedload(trainwreck.TransactionItem.discounts))
+
+        return query
+
+    def normalize_local_object(self, item):
+        data = super(TransactionItemImporter, self).normalize_local_object(item)
+        if not data:
+            return
+
+        if 'transaction_system_id' in self.fields:
+            data['transaction_system_id'] = item.transaction.system_id
+
+        if 'discounts' in self.fields:
+            data['discounts'] = [Discount(d.discount_type, d.description, d.amount)
+                                 for d in item.discounts]
+
+        return data
 
     def get_transaction_by_system_id(self, system_id):
         if hasattr(self, 'transactions_by_system_id'):
@@ -192,3 +213,75 @@ class TransactionItemImporter(ToTrainwreck):
                 item.transaction = txn
 
             return item
+
+    def update_object(self, item, host_data, local_data=None, **kwargs):
+        item = super(TransactionItemImporter, self).update_object(
+            item, host_data, local_data=local_data, **kwargs)
+        if not item:
+            return
+
+        if 'discounts' in self.fields:
+            trainwreck = self.config.get_trainwreck_model()
+            if host_data['discounts']:
+                if not local_data or local_data['discounts'] != host_data['discounts']:
+                    # TODO: is this too heavy-handed?
+                    # just rebuild the line item's "other" discount list,
+                    # instead of trying to reconcile the host vs. local lists
+                    if item.discounts:
+                        del item.discounts[:]
+                    for disc in host_data['discounts']:
+                        item.discounts.append(
+                            trainwreck.TransactionItemDiscount(
+                                discount_type=disc.discount_type,
+                                description=disc.description,
+                                amount=disc.amount))
+            else:
+                if local_data and local_data['discounts']:
+                    del item.discounts[:]
+        return item
+
+
+class TransactionItemDiscountImporter(ToTrainwreck):
+    """
+    Transaction item discount data importer
+
+    .. attribute:: match_on_time_field
+
+       This time field will be used for the cache query etc.  In
+       particular it controls which transactions are deemed to belong
+       to a given date, which is needed when restricting the import to
+       a particular date range.  Can set this to ``'upload_time'`` or
+       ``'start_time'`` if necessary, depending on the nature of
+       transaction data coming from the host side.
+    """
+    match_on_time_field = 'end_time'
+
+    @property
+    def importing_from_system(self):
+        raise NotImplementedError("TODO: please define this for your subclass")
+
+    def get_model_class(self):
+        if hasattr(self, 'model_class') and self.model_class:
+            return self.model_class
+        trainwreck = self.config.get_trainwreck_model()
+        return trainwreck.TransactionItemDiscount
+
+    def cache_query(self):
+        query = super(TransactionItemDiscountImporter, self).cache_query()
+        trainwreck = self.config.get_trainwreck_model()
+
+        query = query.join(trainwreck.TransactionItem)\
+                     .join(trainwreck.Transaction)
+
+        try:
+            system = self.importing_from_system
+        except NotImplementedError:
+            pass
+        else:
+            query = query.filter(trainwreck.Transaction.system == system)
+
+        time_field = getattr(trainwreck.Transaction, self.match_on_time_field)
+        query = query.filter(time_field >= self.app.make_utc(self.start_time))\
+                     .filter(time_field < self.app.make_utc(self.end_time))
+
+        return query
