@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -65,7 +65,8 @@ def capture_output(*args, **kwargs):
     return output
 
 
-def data_diffs(local_data, host_data, fields=None):
+def data_diffs(local_data, host_data, fields=None,
+               fuzzy_fields=None, fuzz_factor=None):
     """
     Find all (relevant) fields which differ between the host and local data
     values for a given record.
@@ -74,6 +75,7 @@ def data_diffs(local_data, host_data, fields=None):
         fields = list(local_data)
 
     diffs = []
+    fuzzy_time_cutoff = datetime.timedelta(seconds=fuzz_factor or 1)
     for field in fields:
 
         # maybe raise explicit error to indicate *which* dict is missing data
@@ -83,7 +85,44 @@ def data_diffs(local_data, host_data, fields=None):
         if field not in host_data:
             raise KeyError("key '{}' is missing from host_data".format(field))
 
-        if local_data[field] != host_data[field]:
+        local_value = local_data[field]
+        host_value = host_data[field]
+
+        if fuzzy_fields and field in fuzzy_fields:
+
+            if local_value and not host_value:
+                diffs.append(field)
+                continue
+
+            if host_value and not local_value:
+                diffs.append(field)
+                continue
+
+            if isinstance(local_value, datetime.datetime):
+
+                if (host_value > local_value
+                    and host_value - local_value >= fuzzy_time_cutoff):
+                    diffs.append(field)
+
+                elif (local_value > host_value
+                      and local_value - host_value >= fuzzy_time_cutoff):
+                    diffs.append(field)
+
+                continue
+
+            elif isinstance(local_value, (int, float, decimal.Decimal)):
+
+                if (host_value > local_value
+                    and host_value - local_value >= fuzz_factor):
+                    diffs.append(field)
+
+                elif (local_value > host_value
+                      and local_value - host_value >= fuzz_factor):
+                    diffs.append(field)
+
+                continue
+
+        if local_value != host_value:
             diffs.append(field)
 
     return diffs
